// Ansteuerung eines LCD 162 im 4-Bit-Interfacemodus am Port C

#include "lcd_162.h"

// Hier die verwendete Taktfrequenz in Hz eintragen, wichtig!
#define F_CPU 1000000
 
#include <avr/io.h>
#include <util/delay.h>

// sendet ein Datenbyte an das LCD
void lcd_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp1;               // setzen
   lcd_enable();
 
   temp2 = temp2 & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp2;               // setzen
   lcd_enable();
   
   _delay_us(42);
}
 
// sendet einen Befehl an das LCD
 
void lcd_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;            // maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp1;               // setzen
   lcd_enable();
 
   temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp2;               // setzen
   lcd_enable();
   
   _delay_us(42);
}
 
// erzeugt den Enable-Puls
void lcd_enable(void)
{
   LCD_PORT |= (1<<LCD_EN);
   _delay_us(1);                   // kurze Pause
   LCD_PORT &= ~(1<<LCD_EN);
}
 
// Initialisierung: 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd_init(void)
{

   _delay_ms(50);
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x03;            
   LCD_PORT &= ~(1<<LCD_RS);      // RS auf 0
   _delay_ms(5);
   lcd_enable();
   _delay_ms(5);
   lcd_enable();
   _delay_ms(1);
   lcd_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x02;
   lcd_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen
   lcd_command(0x28);

   // Display ein / Cursor aus / kein Blinken
   lcd_command(0x0C); 
   
   // Auto-Inkrement / kein Scrollen
   lcd_command(0x06);

   lcd_clear();

}
  
// Sendet den Befehl zur Lschung des Displays
 
void lcd_clear(void)
{
   lcd_command(0x01);
   _delay_ms(2);
}
 
// Sendet den Befehl: Cursor Home
/*
void lcd_home(void)
{
   lcd_command(0x02);
   _delay_ms(2);
}
*/

// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
 
void lcd_set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd_command(tmp);
}
 
// Schreibt einen String auf das LCD
 
void lcd_string(char *data)
{
    while(*data) {
        lcd_data(*data);
        data++;
    }
}

//	Steuert die Displayfunktionen

void lcd_dcb(unsigned char display, unsigned char cursor, unsigned char blinking)
{
	unsigned char temp1 = 0x08;
	
	if (display != 0)
		temp1 |= 0x04;

	if (cursor != 0)
		temp1 |= 0x02;

	if (blinking != 0)
		temp1 |= 0x01;

	lcd_command(temp1);
	
}
